
addpath('./util');
addpath('./gui');

%params
imgw = 25;
k = 8;
%listfile = 'all_cropped_list';
%listfile = 'imglists/smiling_cropped_list';
listfile = 'imglists/nonsmiling_cropped_list';

if ~exist('imgcell', 'var')
	fprintf('loading images from %s ...', listfile);
	imgfilenames = load_text(listfile);
	imgcell = load_images(imgfilenames);
	nimgs = length(imgcell);
	fprintf(' read %d images. DONE.\n', nimgs);
end

if 0
	figure(1);
	set(1,'DoubleBuffer','on');
	for i = 1:nimgs
		image(imgcell{i});
		drawnow;
	end
end

asp = [];
for i = 1:nimgs
	[tmpw,tmph,foo] = size(imgcell{i});
	asp(i) = tmpw/tmph;
end
mean_asp = mean(asp);

imgh = round(imgw/mean_asp);

C_orig = massageImgCell(imgcell,[imgw imgh 3], 0, 1, 0, 2^16-1);
G_orig = massageImgCell(imgcell,[imgw imgh 1], 0, 1, 0, 2^16-1);

if 0
	figure(1);
	set(1,'DoubleBuffer','on');
	colormap(gray(1024));
	for i=1:nimgs, 
		subplot(121); imagesc(reshape(G_orig(:,i),[imgw imgh]));
		subplot(122); image(reshape(C_orig(:,i),[imgw imgh 3])); 
		drawnow; 
	end
end

G_mean = mean(G_orig')';
G = G_orig - repmat(G_mean,1,nimgs);
C_mean = mean(C_orig')';
C = C_orig - repmat(C_mean,1,nimgs);

if 0
	figure(2);
	set(2,'DoubleBuffer','on');
	colormap(gray(1024));
	for i=1:nimgs, 
		imagesc(reshape(G(:,i),[imgw imgh]));
		pause(0.5);
		drawnow; 
	end
end

if 0
	Cov = G*G';
	[PC_full,D] = eig(Cov);
	PC_full = fliplr(PC_full);
else
	[PC_full,S,V] = svd(G,0);
end

PC = PC_full(:,1:k);
PC_inv = inv(PC'*PC)*PC';
G_proj = PC_inv*G;
G_recon = PC*G_proj;
G_recon_mse = mean(sum((G_recon-G).^2,2));

%contstruct model
G_pca = [];
G_pca.is_PCA = 1;
G_pca.w = imgw;
G_pca.h = imgh;
G_pca.mu = G_mean;
G_pca.PC = PC;
G_pca.PC_inv = PC_inv;

if 0
	set(figure(3),'DoubleBuffer','on');
	colormap(gray(1024));
	clf;
	for i=1:k
		subplot(4,5,i);
		imagesc(reshape(PC(:,i), [imgw imgh]));
		drawnow;
	end
end

if 0
	[C_pc,S,D] = svd(C,0);

	set(figure(3),'DoubleBuffer','on');
	colormap(gray(1024));
	clf;
	C_pc_adj = C_pc - min(min(C_pc));
	C_pc_adj = C_pc_adj / max(max(C_pc_adj));
	for i=1:k
		subplot(4,5,i);
		imagesc(reshape(C_pc_adj(:,i), [imgw imgh 3]));
		drawnow;
	end
end

if 0
	set(figure(4),'DoubleBuffer','on');
	colormap(gray(1024));
	clf;
	x = 5*rand(k,1);
	for i=1:1000
		x = x + 2*(rand(k,1)-0.5);
		%x = x/norm(x);
		recon_img = G_mean + PC(:,1:k)*x;
		imagesc(reshape(recon_img, [imgw imgh]));
		drawnow;
		pause(1/25);
	end
end

I_cell = load_images({'../imgdata/group1.tiff'});
I_size = size(I_cell{1});
IC = massageImgCell(I_cell, [I_size(1:2) 3], 0, 1, 0, 2^16-1);
IG = massageImgCell(I_cell, [I_size(1:2) 1], 0, 1, 0, 2^16-1);
n = 5; 
%profile on;
%scales = 1.0;
scales = 1.0;
%scales = 0.5:0.1:1.0;
[Pos,Pos_vals] = find_faces(reshape(IG,I_size(1:2)), G_pca, scales, n, 0);
%profile off;

figure(5);
colormap(gray(1024));
clf;
image(reshape(IC,I_size));
for i=1:n
	lh = draw_rect(Pos(:,i));
	set(lh,'Color',[0 1 0]);
	text(Pos(1,i), Pos(2,i)-5, num2str(i), 'Color', [0 1 0]);
end	


