function [Pos,Pos_vals] = find_faces(I, model, scales, n, allow_overlap)

if ~exist('allow_overlap', 'var')
	allow_overlap = 0;
end

%get image width/height
[iw ih] = size(I);

%this will be used to store the best face positions
% in [x y w h]' format
Pos = [];
%this will either be the mse (for PCA) or loglik (for SPCA)
Pos_vals = [];

%loop over each scale
for scalefact = scales
	
	%resample the image
	Is = resample_img(I, scalefact);

	[sw sh] = size(Is);

	%compute the true width/height of a patch in this sampled image
	pixw = model.w/scalefact;
	pixh = model.w/scalefact;

	%loop over every possible position in Is
	l = model.w*model.h;
	wlist=1:(sw-model.w+1);
	hlist=1:(sh-model.h+1);
	P = zeros(l,length(wlist));
	for j=hlist

		if ~mod(j,10), fprintf('\rScale %1.2f - processing row [%d/%d]', scalefact, j, hlist(end)); end

		%to speed things up all patches that start in this row
		%are copied into a matrix to take advantage of vectorized operations
		for i=wlist
			P(:,i) = reshape(Is(i:(i+model.w-1),j:(j+model.h-1)),[],1) - model.mu;
		end

		%first project into face space
		X = model.PC_inv*P;

		if model.is_PCA
			%now reconstruct
			P_recon = model.PC*X;

			%compute mse
			mse = mean((P-P_recon).^2);

			%sort it
			[sortmse,sortidx] = sort(mse);
			nidx = sortidx(1:n);

			%tack the n best here onto Pos
			Pos(:,(end+1):(end+n)) = [ones(1,n)*(j/scalefact); nidx/scalefact; ones(1,n)*pixw; ones(1,n)*pixh];
			Pos_vals((end+1):(end+n)) = sortmse(1:n);
		else
		end

	end %loop over all rows in the image
	fprintf('\n');

end %loop over all scales

%now sort all the candidates
if model.is_PCA
	[sortmse,sortidx] = sort(Pos_vals);
else
	[sortll,sortidx] = sort(-Pos_vals);
end

if allow_overlap 
	Pos = Pos(:,sortidx(1:n));
	Pos_vals = Pos_vals(sortidx(1:n));
else 
	%now need to find best n non-overlapping
	Pos_best = [];
	Pos_best_vals = [];
	idx = 0;
	for i=1:n
		while 1
			idx = idx + 1;
			overlap = 0;
			for j=1:size(Pos_best,2)
				if rect_overlap(Pos_best(:,j), Pos(:,sortidx(idx)))
					overlap = 1;
					break; 
				end
			end
			if ~overlap, break; end
		end
		Pos_best(:,end+1) = Pos(:,sortidx(idx));
		Pos_best_vals(end+1) = Pos_vals(sortidx(idx));
	end
	Pos = Pos_best;
	Pos_vals = Pos_best_vals;
end


