function learn_pca(k)

global TrainData;
global Model;

Model = [];
Model.is_PCA = 1;

%copy some basic stuff from the training data
Model.imgw = TrainData.imgw;
Model.imgh = TrainData.imgh;
Model.mu = TrainData.mu;

% MAIN COMPUTATION PART
if 0
	%slow way - traditional, compute covariance, do eigen decomp.
	Cov = TrainData.I*TrainData.I';
	[PC_full,S] = eig(Cov);
	Model.PC_full = fliplr(PC_full);
else
	%fast way - simply do SVD on training data
	[PC_full,S,V] = svd(TrainData.I,0);
	Model.PC_full = PC_full;
end

if k < 1, k = size(Model.PC_full,2); end
if k > size(Model.PC_full,2),
	error('Asking for more princ. components than data points!');
	Model = [];
	return;
end
Model.k = k;

%here just pick the k best pric. comps.
Model.PC = Model.PC_full(:,1:k);

%compute the psuedo inverse (very close to the transpose)
Model.PC_inv = inv(Model.PC'*Model.PC)*Model.PC';

%project the face images into face space (X)
TrainData.X = Model.PC_inv*TrainData.I;
TrainData.I_recon = Model.PC*TrainData.X;
TrainData.I_recon_mse = mean(sum((TrainData.I_recon - TrainData.I).^2,2));

% GUI stuff from here to end
clf(figure(1));
colormap(gray(1024));
build_menu;
nr = round(sqrt(k+2));
nc = ceil((k+2)/nr);
subplot(111); %do a reset of subplots
subplot(nr,nc,1);
show_vectimg(Model.mu, [Model.imgw Model.imgh]);
title('Mean face');
idx = 2;
for i=1:k
	subplot(nr,nc,idx);
	show_vectimg(Model.PC(:,i), [Model.imgw Model.imgh]);
	title(sprintf('PC %d',i));
	idx = idx + 1;
end
subplot(nr,nc,idx);
diagS = diag(S);
bar(diagS(1:k));
title('Eigenvalues');
axis([0 k+1 min(diagS(1:k))*0.95 max(diagS(1:k))*1.05]);


